Package: lidR
Type: Package
Title: Airborne LiDAR Data Manipulation and Visualization for Forestry
        Applications
Version: 4.3.2
Authors@R: c(
    person("Jean-Romain", "Roussel", email = "info@r-lidar.com", role = c("aut", "cre", "cph")),
    person("David", "Auty", email = "", role = c("aut", "ctb"), comment = "Reviews the documentation"),
    person("Florian", "De Boissieu", email = "", role = ("ctb"), comment = "Fixed bugs and improved catalog features"),
    person("Andrew", "Sánchez Meador", email = "", role = ("ctb"), comment = "Implemented wing2015() for segment_snags()"),
    person("Bourdon", "Jean-François", email = "", role = ("ctb"), comment = "Contributed to Roussel2020() for track_sensor()"),
    person("Gatziolis", "Demetrios", email = "", role = ("ctb"), comment = "Implemented Gatziolis2019() for track_sensor()"),
    person("Leon", "Steinmeier", email = "", role = ("ctb"), comment = "Contributed to parallelization management"),
    person("Stanislaw", "Adaszewski", email = "", role = ("cph"), comment = "Author of the C++ concaveman code"),
    person("Benoît", "St-Onge", email = "", role = "cph", comment = "Author of the 'chm_prep' function"))
Description: Airborne LiDAR (Light Detection and Ranging) interface for data
    manipulation and visualization. Read/write 'las' and 'laz' files, computation
    of metrics in area based approach, point filtering, artificial point reduction,
    classification from geographic data, normalization, individual tree segmentation
    and other manipulations.
URL: https://github.com/r-lidar/lidR
BugReports: https://github.com/r-lidar/lidR/issues
License: GPL-3
Depends: R (>= 3.5.0), methods
Imports: classInt, data.table (>= 1.12.0), glue, grDevices, lazyeval,
        Rcpp (>= 1.0.3), rgl, rlas (>= 1.8.2), sf, stats, stars, terra
        (>= 1.5-17), tools, utils, parallelly
Suggests: EBImage, future, geometry, gstat, raster, RCSF, RMCC, rjson,
        mapview, mapedit, progress, sp, testthat (>= 2.1.0), knitr,
        rmarkdown
RoxygenNote: 7.3.3
LinkingTo: BH (>= 1.72.0),Rcpp,RcppArmadillo
Encoding: UTF-8
ByteCompile: true
VignetteBuilder: knitr
Biarch: true
Collate: 'Class-LAS.R' 'Class-LAScatalog.R' 'Class-LAScluster.R'
        'RcppExports.R' 'add_attribute.R' 'algorithm-dec.R'
        'algorithm-dsm.R' 'algorithm-dtm.R' 'algorithm-gnd.R'
        'algorithm-itd.R' 'algorithm-its.R' 'algorithm-noi.R'
        'algorithm-out.R' 'algorithm-shp.R' 'algorithm-snag.R'
        'algorithm-trk.R' 'backward_compatibility.R' 'catalog_apply.R'
        'catalog_boundaries.R' 'catalog_fakerun.R'
        'catalog_intersect.R' 'catalog_laxindex.R' 'catalog_overlaps.R'
        'catalog_retile.R' 'catalog_select.R' 'classify.R'
        'classify_ground.R' 'classify_noise.R' 'classify_poi.R'
        'clip_roi.R' 'connected_components.R' 'decimate_points.R'
        'io_readLAScatalog.R' 'io_readXLAS.R' 'deprecated.R'
        'doc-drivers.R' 'doc-lidR.R' 'doc-parallelism.R'
        'doc-spatialindex.R' 'engine.R' 'engine_apply.R'
        'engine_chunks.R' 'engine_crop.R' 'engine_index.R'
        'engine_merge.R' 'engine_options.R' 'engine_write.R'
        'fasterize.R' 'filters.R' 'fit_shapes.R' 'fullwaveform.R'
        'generate_las.R' 'io_readLAS.R' 'io_writeLAS.R' 'knn.R'
        'las_check.R' 'las_compression.R' 'las_tools.R'
        'locate_localmaxima.R' 'locate_trees.R' 'merge_spatial.R'
        'methods-LAS.R' 'methods-LAScatalog.R' 'methods-LAScluster.R'
        'methods-LASheader.R' 'metrics_cloud.R' 'metrics_crowns.R'
        'metrics_hexagons.R' 'metrics_pixels.R' 'metrics_plot.R'
        'metrics_point.R' 'metrics_polygon.R' 'metrics_stdmetrics.R'
        'metrics_template.R' 'metrics_voxels.R' 'normalize.R'
        'normalize_height.R' 'normalize_intensity.R' 'plot.R'
        'plot.s3.R' 'plugins.R' 'print.R' 'rasterize.R'
        'rasterize_canopy.R' 'rasterize_density.R'
        'rasterize_terrain.R' 'retrieve_info.R' 'segment.R'
        'segment_shapes.R' 'segment_snags.R' 'segment_trees.R'
        'smooth_height.R' 'st_area.R' 'st_as_sf.R' 'st_bbox.R'
        'st_coordinates.R' 'st_crs.R' 'st_hull.R' 'st_misc.R'
        'st_transform.R' 'track_sensor.R' 'utils_assertive.R'
        'utils_base.R' 'utils_chm.R' 'utils_colors.R'
        'utils_constant.R' 'utils_delaunay.R' 'utils_geometry.R'
        'utils_is.R' 'utils_misc.R' 'utils_raster.R'
        'utils_spatial_index.R' 'utils_threads.R' 'voxelize_points.R'
        'zzz.R'
NeedsCompilation: yes
Packaged: 2026-04-21 11:21:52 UTC; jr
Author: Jean-Romain Roussel [aut, cre, cph],
  David Auty [aut, ctb] (Reviews the documentation),
  Florian De Boissieu [ctb] (Fixed bugs and improved catalog features),
  Andrew Sánchez Meador [ctb] (Implemented wing2015() for
    segment_snags()),
  Bourdon Jean-François [ctb] (Contributed to Roussel2020() for
    track_sensor()),
  Gatziolis Demetrios [ctb] (Implemented Gatziolis2019() for
    track_sensor()),
  Leon Steinmeier [ctb] (Contributed to parallelization management),
  Stanislaw Adaszewski [cph] (Author of the C++ concaveman code),
  Benoît St-Onge [cph] (Author of the 'chm_prep' function)
Maintainer: Jean-Romain Roussel <info@r-lidar.com>
Repository: CRAN
Date/Publication: 2026-04-21 12:32:08 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-21 19:20:09 UTC; unix
Archs: lidR.so.dSYM
