# Generated by roxygen2: do not edit by hand

S3method("st_crs<-",LAS)
S3method("st_crs<-",LAScatalog)
S3method("st_crs<-",LASheader)
S3method(as.list,LASheader)
S3method(as.spatial,LAS)
S3method(as.spatial,LAScatalog)
S3method(classify_ground,LAS)
S3method(classify_ground,LAScatalog)
S3method(classify_noise,LAS)
S3method(classify_noise,LAScatalog)
S3method(classify_poi,LAS)
S3method(classify_poi,LAScatalog)
S3method(clip_circle,LAS)
S3method(clip_circle,LAScatalog)
S3method(clip_rectangle,LAS)
S3method(clip_rectangle,LAScatalog)
S3method(cloud_metrics,LAS)
S3method(cloud_metrics,LAScluster)
S3method(crown_metrics,LAS)
S3method(crown_metrics,LAScatalog)
S3method(crown_metrics,LAScluster)
S3method(decimate_points,LAS)
S3method(decimate_points,LAScatalog)
S3method(dim,LAS)
S3method(dim,LAScatalog)
S3method(engine_crop,LAS)
S3method(engine_crop,Raster)
S3method(engine_crop,SpatRaster)
S3method(engine_crop,Spatial)
S3method(engine_crop,default)
S3method(engine_crop,sf)
S3method(engine_crop,sfc)
S3method(engine_crop,stars)
S3method(filter_duplicates,LAS)
S3method(filter_duplicates,LAScatalog)
S3method(filter_surfacepoints,LAS)
S3method(filter_surfacepoints,LAScatalog)
S3method(hexagon_metrics,LAS)
S3method(hexagon_metrics,LAScluster)
S3method(las_check,LAS)
S3method(las_check,LAScatalog)
S3method(las_check,LAScluster)
S3method(las_check,LASheader)
S3method(locate_trees,LAS)
S3method(locate_trees,LAScatalog)
S3method(locate_trees,RasterLayer)
S3method(locate_trees,SpatRaster)
S3method(locate_trees,stars)
S3method(merge_spatial,LAS)
S3method(names,LAS)
S3method(names,LAScatalog)
S3method(names,LASheader)
S3method(normalize_height,LAS)
S3method(normalize_height,LAScatalog)
S3method(normalize_intensity,LAS)
S3method(normalize_intensity,LAScatalog)
S3method(npoints,LAS)
S3method(npoints,LAScatalog)
S3method(npoints,LASheader)
S3method(pixel_metrics,LAS)
S3method(pixel_metrics,LAScatalog)
S3method(pixel_metrics,LAScluster)
S3method(plot,lasmetrics3d)
S3method(point_metrics,LAS)
S3method(polygon_metrics,LAS)
S3method(print,LAS)
S3method(print,LAScatalog)
S3method(print,lidRAlgorithm)
S3method(print,raster_template)
S3method(rasterize_canopy,LAS)
S3method(rasterize_canopy,LAScatalog)
S3method(rasterize_canopy,LAScluster)
S3method(rasterize_density,LAS)
S3method(rasterize_density,LAScatalog)
S3method(rasterize_terrain,LAS)
S3method(rasterize_terrain,LAScatalog)
S3method(rasterize_terrain,LAScluster)
S3method(rbind,LAS)
S3method(rbind,LAScatalog)
S3method(readLAS,LAScatalog)
S3method(readLAS,LAScluster)
S3method(readLAS,character)
S3method(rumple_index,RasterLayer)
S3method(rumple_index,SpatRaster)
S3method(rumple_index,matrix)
S3method(rumple_index,numeric)
S3method(rumple_index,stars)
S3method(segment_shapes,LAS)
S3method(segment_shapes,LAScatalog)
S3method(segment_snags,LAS)
S3method(segment_snags,LAScatalog)
S3method(segment_trees,LAS)
S3method(segment_trees,LAScatalog)
S3method(st_area,LAS)
S3method(st_area,LAScatalog)
S3method(st_area,LASheader)
S3method(st_as_sf,LAS)
S3method(st_as_sf,LAScatalog)
S3method(st_bbox,LAS)
S3method(st_bbox,LAScatalog)
S3method(st_bbox,LAScluster)
S3method(st_bbox,LASheader)
S3method(st_concave_hull,LAS)
S3method(st_convex_hull,LAS)
S3method(st_coordinates,LAS)
S3method(st_coordinates,LAScatalog)
S3method(st_crs,LAS)
S3method(st_crs,LAScatalog)
S3method(st_crs,LAScluster)
S3method(st_crs,LASheader)
S3method(st_transform,LAS)
S3method(summary,LAS)
S3method(summary,LAScatalog)
S3method(template_metrics,LAS)
S3method(track_sensor,LAS)
S3method(track_sensor,LAScatalog)
S3method(track_sensor,LAScluster)
S3method(voxelize_points,LAS)
S3method(voxelize_points,LAScatalog)
S3method(voxelize_points,LAScluster)
export("epsg<-")
export("index<-")
export("opt_chunk_alignment<-")
export("opt_chunk_buffer<-")
export("opt_chunk_size<-")
export("opt_filter<-")
export("opt_independent_files<-")
export("opt_laz_compression<-")
export("opt_merge<-")
export("opt_output_files<-")
export("opt_progress<-")
export("opt_restart<-")
export("opt_select<-")
export("opt_stop_early<-")
export("opt_wall_to_wall<-")
export("projection<-")
export("sensor<-")
export("st_crs<-")
export("wkt<-")
export(.stdmetrics)
export(.stdmetrics_ctrl)
export(.stdmetrics_i)
export(.stdmetrics_pulse)
export(.stdmetrics_rn)
export(.stdmetrics_z)
export(.stdshapemetrics)
export(.stdtreemetrics)
export(Gatziolis2019)
export(LAD)
export(LAS)
export(LASBRIGDE)
export(LASBUILDING)
export(LASGROUND)
export(LASHIGHVEGETATION)
export(LASKEYPOINT)
export(LASLOWPOINT)
export(LASLOWVEGETATION)
export(LASMEDIUMVEGETATION)
export(LASNOISE)
export(LASNONCLASSIFIED)
export(LASRAIL)
export(LASROADSURFACE)
export(LASTRANSMISSIONTOWER)
export(LASUNCLASSIFIED)
export(LASWATER)
export(LASWIRECONDUCTOR)
export(LASWIREGUARD)
export(LASheader)
export(Roussel2020)
export(VCI)
export(add_attribute)
export(add_circle3d)
export(add_dtm3d)
export(add_flightlines3d)
export(add_lasattribute)
export(add_lasattribute_manual)
export(add_lasnir)
export(add_lasrgb)
export(add_treetops3d)
export(as.spatial)
export(barycenter_per_voxel)
export(catalog)
export(catalog_apply)
export(catalog_boundaries)
export(catalog_intersect)
export(catalog_map)
export(catalog_retile)
export(catalog_sapply)
export(catalog_select)
export(catalog_split_clusters)
export(classify_ground)
export(classify_noise)
export(classify_poi)
export(clip_circle)
export(clip_polygon)
export(clip_rectangle)
export(clip_roi)
export(clip_transect)
export(cloud_metrics)
export(concaveman)
export(connected_components)
export(count_not_quantized)
export(crown_metrics)
export(csf)
export(dalponte2016)
export(decimate_points)
export(delineate_crowns)
export(density)
export(dsmtin)
export(engine_apply)
export(engine_chunks)
export(engine_crop)
export(engine_merge)
export(engine_write)
export(entropy)
export(epsg)
export(evlr)
export(filter_duplicates)
export(filter_first)
export(filter_firstlast)
export(filter_firstofmany)
export(filter_ground)
export(filter_last)
export(filter_nth)
export(filter_poi)
export(filter_single)
export(filter_surfacepoints)
export(find_trees)
export(fit_circle)
export(forest.colors)
export(gap_fraction_profile)
export(get_lidr_threads)
export(get_range)
export(grid_canopy)
export(grid_density)
export(grid_metrics)
export(grid_terrain)
export(header)
export(height.colors)
export(height_above_ground)
export(hexagon_metrics)
export(highest)
export(highest_attribute_per_voxel)
export(homogenize)
export(index)
export(interpret_waveform)
export(is)
export(is.algorithm)
export(is.indexed)
export(is.overlapping)
export(is.parallelised)
export(is.quantized)
export(ivf)
export(knn)
export(knn_distance)
export(knnidw)
export(knnx)
export(kriging)
export(las_check)
export(las_is_compressed)
export(las_quantize)
export(las_reoffset)
export(las_rescale)
export(las_size)
export(las_update)
export(li2012)
export(lmf)
export(locate_trees)
export(lowest)
export(lowest_attribute_per_voxel)
export(lspikefree)
export(manual)
export(mcc)
export(merge_spatial)
export(ncol.LAS)
export(normalize_height)
export(normalize_intensity)
export(npoints)
export(nrow.LAScatalog)
export(opt_chunk_alignment)
export(opt_chunk_buffer)
export(opt_chunk_size)
export(opt_filter)
export(opt_independent_files)
export(opt_laz_compression)
export(opt_merge)
export(opt_output_files)
export(opt_progress)
export(opt_select)
export(opt_stop_early)
export(opt_wall_to_wall)
export(p2r)
export(pastel.colors)
export(payload)
export(phb)
export(pitfill_stonge2008)
export(pitfree)
export(pixel_metrics)
export(plot)
export(plot_dtm3d)
export(plot_metrics)
export(plugin_decimate)
export(plugin_dsm)
export(plugin_dtm)
export(plugin_gnd)
export(plugin_itd)
export(plugin_its)
export(plugin_nintensity)
export(plugin_outliers)
export(plugin_shape)
export(plugin_snag)
export(plugin_track)
export(pmf)
export(point_eigenvalues)
export(point_metrics)
export(polygon_metrics)
export(projection)
export(ptd)
export(quantize)
export(random)
export(random.colors)
export(random_per_voxel)
export(range_correction)
export(rasterize_canopy)
export(rasterize_density)
export(rasterize_terrain)
export(readALS)
export(readALSLAS)
export(readALSLAScatalog)
export(readALScatalog)
export(readDAPLAS)
export(readDAPLAScatalog)
export(readLAS)
export(readLAScatalog)
export(readLASheader)
export(readMSLAS)
export(readTLS)
export(readTLSLAS)
export(readTLSLAScatalog)
export(readTLScatalog)
export(readUAVLAS)
export(readUAVLAScatalog)
export(remove_ground)
export(remove_lasattribute)
export(remove_noise)
export(remove_water)
export(retrieve_flightlines)
export(retrieve_pulses)
export(retrieve_scanlines)
export(rumple_index)
export(segment_shapes)
export(segment_snags)
export(segment_trees)
export(sensor)
export(set_lidr_threads)
export(shp_hline)
export(shp_hplane)
export(shp_line)
export(shp_plane)
export(shp_vline)
export(silva2016)
export(smooth_height)
export(sor)
export(spikefree)
export(st_area)
export(st_bbox)
export(st_concave_hull)
export(st_crs)
export(stdmetrics)
export(stdmetrics_ctrl)
export(stdmetrics_i)
export(stdmetrics_pulse)
export(stdmetrics_rn)
export(stdmetrics_z)
export(stdshapemetrics)
export(stdtreemetrics)
export(storable_coordinate_range)
export(template_metrics)
export(tin)
export(track_sensor)
export(tree_metrics)
export(unnormalize_height)
export(unsmooth_height)
export(util_makeZhangParam)
export(vlr)
export(voxel_metrics)
export(voxelize_points)
export(watershed)
export(wing2015)
export(wkt)
export(writeLAS)
exportClasses(LAS)
exportClasses(LAScatalog)
exportClasses(LASheader)
exportMethods("$")
exportMethods("$<-")
exportMethods("-")
exportMethods("[")
exportMethods("[[")
exportMethods("[[<-")
exportMethods("crs<-")
exportMethods("epsg<-")
exportMethods("wkt<-")
exportMethods(area)
exportMethods(crs)
exportMethods(density)
exportMethods(epsg)
exportMethods(ext)
exportMethods(is.empty)
exportMethods(plot)
exportMethods(wkt)
import(Rcpp)
import(data.table)
import(methods)
importFrom(data.table,":=")
importFrom(sf,"st_crs<-")
importFrom(sf,st_area)
importFrom(sf,st_as_sf)
importFrom(sf,st_bbox)
importFrom(sf,st_convex_hull)
importFrom(sf,st_coordinates)
importFrom(sf,st_crs)
importFrom(sf,st_transform)
importFrom(stats,na.omit)
importFrom(terra,area)
importFrom(terra,ext)
importFrom(terra,is.empty)
importMethodsFrom(terra,"crs<-")
importMethodsFrom(terra,crs)
useDynLib(lidR, .registration = TRUE)
