#' Functions for the LAScatalog processing engine not meant to be called directly by users
#'
#' Functions for the LAScatalog processing engine not meant to be called directly by users.
#' They are exported for debugging and to simplify export of internal functions when processing in
#' parallel
#'
#' @param ctg an object of class \code{LAScatalog}
#' @param x LAS, Raster, stars, SpatRaster,sf, sfc, Spatial
#' @name engine
#' @rdname engine
#' @family LAScatalog processing engine
NULL
